/*
Copyright [2020] [https://www.xiaonuo.vip]

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：

1.请不要删除和修改根目录下的LICENSE文件。
2.请不要删除和修改Snowy源码头部的版权声明。
3.请保留源码和相关描述文件的项目出处，作者声明等。
4.分发源码时候，请注明软件出处 https://gitee.com/xiaonuobase/snowy
5.在修改包名，模块名称，项目代码等时，请注明软件出处 https://gitee.com/xiaonuobase/snowy
6.若您的项目无法满足以上几点，可申请商业授权，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.huimv.modular.alarmrecord.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.core.annotion.BusinessLog;
import com.huimv.core.annotion.Permission;
import com.huimv.core.context.login.LoginContextHolder;
import com.huimv.core.enums.LogAnnotionOpTypeEnum;
import com.huimv.core.pojo.response.ResponseData;
import com.huimv.core.pojo.response.SuccessResponseData;
import com.huimv.modular.alarmrecord.entity.AlarmRecord;
import com.huimv.modular.alarmrecord.mapper.AlarmRecordMapper;
import com.huimv.modular.alarmrecord.param.AlarmRecordParam;
import com.huimv.modular.alarmrecord.service.AlarmRecordService;
import com.huimv.modular.alarmvalueset.mapper.AlarmValueSetMapper;
import com.huimv.modular.alarmvalueset.service.AlarmValueSetService;
import com.huimv.modular.collectorregister.entity.CollectorRegister;
import com.huimv.modular.collectorregister.mapper.CollectorRegisterMapper;
import com.huimv.modular.farmalarmnumber.entity.FarmAlarmNumber;
import com.huimv.modular.farmalarmnumber.mapper.FarmAlarmNumberMapper;
import com.huimv.modular.utils.util.StringUtilsWork;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 报警记录控制器
 *
 * @author ydd
 * @date 2021-08-20 13:27:00
 */
@RestController
public class AlarmRecordController {



    @Resource
    private AlarmRecordService alarmRecordService;
    @Resource
    private AlarmValueSetService alarmValueSetService;
    @Resource
    private CollectorRegisterMapper collectorRegisterMapper;
    @Resource
    private AlarmValueSetMapper alarmValueSetMapper;
    @Resource
    private AlarmRecordMapper alarmRecordMapper;
    @Resource
    private FarmAlarmNumberMapper farmAlarmNumberMapper;

    /**
     * 添加报警参数设置
     *
     * @author ydd
     * @date 2021-08-20 13:23:30
     */
    //@Permission
    @PostMapping("/alarmRecord/listByOrgid")
    // public ResponseData add(@RequestBody @Validated(AlarmValueSetParam.add.class) AlarmValueSetParam alarmValueSetParam) {
    public Map listByOrgid(@RequestBody Map<String, Object> params) {

        Long orgId = LoginContextHolder.me().getSysLoginUserOrgId();
        List<Long> list = alarmRecordMapper.findChildFarmsByOrgIds(orgId);
        Map<String, Object> map = new HashMap<>();

        Integer earHighTemp = 0;
        Integer earLowTemp = 0;
        Integer envHighTemp = 0;
        Integer envLowTemp = 0;
        Integer deviceAlarm = 0;
        Integer disesAlarm = 0;
        IPage<FarmAlarmNumber> iPage = new Page<>();
        LambdaQueryWrapper<FarmAlarmNumber> wrapper4 = Wrappers.lambdaQuery();
        wrapper4.in(FarmAlarmNumber::getOrgid, list);
        IPage<FarmAlarmNumber> farmAlarmNumberIPage = farmAlarmNumberMapper.selectPage(iPage, wrapper4);
        System.out.println(farmAlarmNumberIPage.getRecords());
        //牧场相加
        for (FarmAlarmNumber farmAlarmNumber : farmAlarmNumberIPage.getRecords()) {
            earHighTemp += farmAlarmNumber.getEarHightemp();
            earLowTemp += farmAlarmNumber.getEarHightemp();
            envHighTemp += farmAlarmNumber.getEnvHightemp();
            envLowTemp += farmAlarmNumber.getEnvLowtemp();
            deviceAlarm += farmAlarmNumber.getDaviceAlarm();
            disesAlarm += farmAlarmNumber.getDiseaseAlarm();
        }
        map.put("earHighTemp", earHighTemp);
        map.put("earLowTemp", earLowTemp);
        map.put("envHighTemp", envHighTemp);
        map.put("envLowTemp", envLowTemp);
        map.put("deviceAlarm", 0);
        map.put("disesAlarm", 0);
        map.put("total", farmAlarmNumberIPage);
        return map;
    }



    /**
     * 添加报警参数设置
     *
     * @author ydd
     * @date 2021-08-20 13:23:30
     */
    //@Permission
    @PostMapping("/alarmRecord/listByfarmOrgid")
    // public ResponseData add(@RequestBody @Validated(AlarmValueSetParam.add.class) AlarmValueSetParam alarmValueSetParam) {
    public Map listByfarmOrgid(@RequestBody Map<String, Object> params) {
        Long FinalOrdId;
        String orgId = (String) params.get("orgId");//开始时间
        Integer pageNum = (Integer) params.get("pageNum");//开始时间
        Integer pageSize = (Integer) params.get("pageSize");//开始时间
        if (StringUtilsWork.isNotBlank(orgId)){
            FinalOrdId=Long.valueOf(orgId);
        }else {
            FinalOrdId=LoginContextHolder.me().getSysLoginUserOrgId();
        }
        Map<String, Object> map = new HashMap<>();
        //拿到组织ID 放到map   //牧场为空，返回省数据或者市数据，不为空返回
        LambdaQueryWrapper<FarmAlarmNumber> wrapper4 = Wrappers.lambdaQuery();
        wrapper4.eq(FarmAlarmNumber::getOrgid, FinalOrdId);
        FarmAlarmNumber farmAlarmNumber = farmAlarmNumberMapper.selectOne(wrapper4);
        IPage<AlarmRecord> iPage = new Page<>(pageNum,pageSize);
        LambdaQueryWrapper<AlarmRecord> wrapper45 = Wrappers.lambdaQuery();
        wrapper45.eq(AlarmRecord::getOrgId, FinalOrdId).orderByDesc(AlarmRecord::getId);
        IPage<AlarmRecord> alarmRecordIPage = alarmRecordMapper.selectPage(iPage, wrapper45);

        for (AlarmRecord record : alarmRecordIPage.getRecords()) {
            LambdaQueryWrapper<CollectorRegister> wrapper48 = Wrappers.lambdaQuery();
            wrapper48.eq(CollectorRegister::getDeviceCode, record.getCollectorCode());
            CollectorRegister collectorRegister = collectorRegisterMapper.selectOne(wrapper48);
            record.setFarmName(collectorRegister.getPenName());
            record.setRemark(collectorRegister.getUnitName());
        }
        map.put("farmAlarmNumber", farmAlarmNumber);
        map.put("alarmRecordIPage", alarmRecordIPage);
        return map;
    }


    /**
     * 查询报警记录
     *
     * @author ydd
     * @date 2021-08-20 13:27:00
     */

    @GetMapping("/alarmRecord/page")
    @BusinessLog(title = "报警记录_查询", opType = LogAnnotionOpTypeEnum.QUERY)
    public ResponseData page(AlarmRecordParam alarmRecordParam) {
        return new SuccessResponseData(alarmRecordService.page(alarmRecordParam));
    }


    /**
     * 添加报警记录
     *
     * @author ydd
     * @date 2021-08-20 13:27:00
     */

    @PostMapping("/alarmRecord/add")
    @BusinessLog(title = "报警记录_增加", opType = LogAnnotionOpTypeEnum.ADD)
    public ResponseData add(@RequestBody @Validated(AlarmRecordParam.add.class) AlarmRecordParam alarmRecordParam) {
        alarmRecordService.add(alarmRecordParam);
        return new SuccessResponseData();
    }

    /**
     * 删除报警记录，可批量删除
     *
     * @author ydd
     * @date 2021-08-20 13:27:00
     */

    @PostMapping("/alarmRecord/delete")
    @BusinessLog(title = "报警记录_删除", opType = LogAnnotionOpTypeEnum.DELETE)
    public ResponseData delete(@RequestBody @Validated(AlarmRecordParam.delete.class) List<AlarmRecordParam> alarmRecordParamList) {
        alarmRecordService.delete(alarmRecordParamList);
        return new SuccessResponseData();
    }

    /**
     * 编辑报警记录
     *
     * @author ydd
     * @date 2021-08-20 13:27:00
     */

    @PostMapping("/alarmRecord/edit")
    @BusinessLog(title = "报警记录_编辑", opType = LogAnnotionOpTypeEnum.EDIT)
    public ResponseData edit(@RequestBody @Validated(AlarmRecordParam.edit.class) AlarmRecordParam alarmRecordParam) {
        alarmRecordService.edit(alarmRecordParam);
        return new SuccessResponseData();
    }

    /**
     * 查看报警记录
     *
     * @author ydd
     * @date 2021-08-20 13:27:00
     */

    @GetMapping("/alarmRecord/detail")
    @BusinessLog(title = "报警记录_查看", opType = LogAnnotionOpTypeEnum.DETAIL)
    public ResponseData detail(@Validated(AlarmRecordParam.detail.class) AlarmRecordParam alarmRecordParam) {
        return new SuccessResponseData(alarmRecordService.detail(alarmRecordParam));
    }

    /**
     * 报警记录列表
     *
     * @author ydd
     * @date 2021-08-20 13:27:00
     */

    @GetMapping("/alarmRecord/list")
    @BusinessLog(title = "报警记录_列表", opType = LogAnnotionOpTypeEnum.QUERY)
    public ResponseData list(AlarmRecordParam alarmRecordParam) {
        return new SuccessResponseData(alarmRecordService.list(alarmRecordParam));
    }

    /**
     * 导出系统用户
     *
     * @author ydd
     * @date 2021-08-20 13:27:00
     */

    @GetMapping("/alarmRecord/export")
    @BusinessLog(title = "报警记录_导出", opType = LogAnnotionOpTypeEnum.EXPORT)
    public void export(AlarmRecordParam alarmRecordParam) {
        alarmRecordService.export(alarmRecordParam);
    }

}
