/*
Copyright [2020] [https://www.xiaonuo.vip]

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：

1.请不要删除和修改根目录下的LICENSE文件。
2.请不要删除和修改Snowy源码头部的版权声明。
3.请保留源码和相关描述文件的项目出处，作者声明等。
4.分发源码时候，请注明软件出处 https://gitee.com/xiaonuobase/snowy
5.在修改包名，模块名称，项目代码等时，请注明软件出处 https://gitee.com/xiaonuobase/snowy
6.若您的项目无法满足以上几点，可申请商业授权，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.huimv.modular.eartagregister.controller;

import com.huimv.core.annotion.BusinessLog;
import com.huimv.core.annotion.Permission;
import com.huimv.core.enums.LogAnnotionOpTypeEnum;
import com.huimv.core.pojo.response.ResponseData;
import com.huimv.core.pojo.response.SuccessResponseData;
import com.huimv.modular.eartagregister.param.EartagRegisterParam;
import com.huimv.modular.eartagregister.service.EartagRegisterService;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import javax.annotation.Resource;
import java.util.List;

/**
 * 耳标注册控制器
 *
 * @author 杨迪
 * @date 2021-08-21 17:19:57
 */
@RestController
public class EartagRegisterController {

    @Resource
    private EartagRegisterService eartagRegisterService;

    /**
     * 查询耳标注册
     *
     * @author 杨迪
     * @date 2021-08-21 17:19:57
     */

    @GetMapping("/eartagRegister/page")
    @BusinessLog(title = "耳标注册_查询", opType = LogAnnotionOpTypeEnum.QUERY)
    public ResponseData page(EartagRegisterParam eartagRegisterParam) {
        return new SuccessResponseData(eartagRegisterService.page(eartagRegisterParam));
    }

    /**
     * 添加耳标注册
     *
     * @author 杨迪
     * @date 2021-08-21 17:19:57
     */

    @PostMapping("/eartagRegister/add")
    @BusinessLog(title = "耳标注册_增加", opType = LogAnnotionOpTypeEnum.ADD)
    public ResponseData add(@RequestBody @Validated(EartagRegisterParam.add.class) EartagRegisterParam eartagRegisterParam) {
            eartagRegisterService.add(eartagRegisterParam);
        return new SuccessResponseData();
    }

    /**
     * 删除耳标注册，可批量删除
     *
     * @author 杨迪
     * @date 2021-08-21 17:19:57
     */

    @PostMapping("/eartagRegister/delete")
    @BusinessLog(title = "耳标注册_删除", opType = LogAnnotionOpTypeEnum.DELETE)
    public ResponseData delete(@RequestBody @Validated(EartagRegisterParam.delete.class) List<EartagRegisterParam> eartagRegisterParamList) {
            eartagRegisterService.delete(eartagRegisterParamList);
        return new SuccessResponseData();
    }

    /**
     * 编辑耳标注册
     *
     * @author 杨迪
     * @date 2021-08-21 17:19:57
     */

    @PostMapping("/eartagRegister/edit")
    @BusinessLog(title = "耳标注册_编辑", opType = LogAnnotionOpTypeEnum.EDIT)
    public ResponseData edit(@RequestBody @Validated(EartagRegisterParam.edit.class) EartagRegisterParam eartagRegisterParam) {
            eartagRegisterService.edit(eartagRegisterParam);
        return new SuccessResponseData();
    }

    /**
     * 查看耳标注册
     *
     * @author 杨迪
     * @date 2021-08-21 17:19:57
     */

    @GetMapping("/eartagRegister/detail")
    @BusinessLog(title = "耳标注册_查看", opType = LogAnnotionOpTypeEnum.DETAIL)
    public ResponseData detail(@Validated(EartagRegisterParam.detail.class) EartagRegisterParam eartagRegisterParam) {
        return new SuccessResponseData(eartagRegisterService.detail(eartagRegisterParam));
    }

    /**
     * 耳标注册列表
     *
     * @author 杨迪
     * @date 2021-08-21 17:19:57
     */

    @GetMapping("/eartagRegister/list")
    @BusinessLog(title = "耳标注册_列表", opType = LogAnnotionOpTypeEnum.QUERY)
    public ResponseData list(EartagRegisterParam eartagRegisterParam) {
        return new SuccessResponseData(eartagRegisterService.list(eartagRegisterParam));
    }

    /**
     * 导出系统用户
     *
     * @author 杨迪
     * @date 2021-08-21 17:19:57
     */

    @GetMapping("/eartagRegister/export")
    @BusinessLog(title = "耳标注册_导出", opType = LogAnnotionOpTypeEnum.EXPORT)
    public void export(EartagRegisterParam eartagRegisterParam) {
        eartagRegisterService.export(eartagRegisterParam);
    }

}
