/*
Copyright [2020] [https://www.xiaonuo.vip]

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：

1.请不要删除和修改根目录下的LICENSE文件。
2.请不要删除和修改Snowy源码头部的版权声明。
3.请保留源码和相关描述文件的项目出处，作者声明等。
4.分发源码时候，请注明软件出处 https://gitee.com/xiaonuobase/snowy
5.在修改包名，模块名称，项目代码等时，请注明软件出处 https://gitee.com/xiaonuobase/snowy
6.若您的项目无法满足以上几点，可申请商业授权，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.huimv.modular.eartagregister.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.core.consts.CommonConstant;
import com.huimv.core.enums.CommonStatusEnum;
import com.huimv.core.exception.ServiceException;
import com.huimv.core.factory.PageFactory;
import com.huimv.core.pojo.page.PageResult;
import com.huimv.core.util.PoiUtil;
import com.huimv.modular.eartagregister.entity.EartagRegister;
import com.huimv.modular.eartagregister.enums.EartagRegisterExceptionEnum;
import com.huimv.modular.eartagregister.mapper.EartagRegisterMapper;
import com.huimv.modular.eartagregister.param.EartagRegisterParam;
import com.huimv.modular.eartagregister.service.EartagRegisterService;
import com.huimv.modular.yesterdaypig.entity.YesterdayPig;
import com.huimv.modular.yesterdaypig.mapper.YesterdayPigMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import javax.annotation.Resource;
import java.util.List;

/**
 * 耳标注册service接口实现类
 *
 * @author 杨迪
 * @date 2021-08-21 17:19:57
 */
@Service
public class EartagRegisterServiceImpl extends ServiceImpl<EartagRegisterMapper, EartagRegister> implements EartagRegisterService {
    @Autowired
    private YesterdayPigMapper yesterdayPigMapper;
    @Transactional
    @Override
    public void updateYestdayPig() {
        List<EartagRegister> list = this.list();
        yesterdayPigMapper.delete(null);
        for (EartagRegister eartagRegister : list) {
            YesterdayPig yesterdayPig = new YesterdayPig();
            BeanUtil.copyProperties(eartagRegister,yesterdayPig);
            yesterdayPigMapper.insert(yesterdayPig);
        }

    }
    @Override
    public PageResult<EartagRegister> page(EartagRegisterParam eartagRegisterParam) {
        QueryWrapper<EartagRegister> queryWrapper = new QueryWrapper<>();
        if (ObjectUtil.isNotNull(eartagRegisterParam)) {

            // 根据组织id 查询
            if (ObjectUtil.isNotEmpty(eartagRegisterParam.getOrgId())) {
                queryWrapper.lambda().eq(EartagRegister::getOrgId, eartagRegisterParam.getOrgId());
            }
            // 根据 查询
            if (ObjectUtil.isNotEmpty(eartagRegisterParam.getEartagNo())) {
                queryWrapper.lambda().eq(EartagRegister::getEartagNo, eartagRegisterParam.getEartagNo());
            }
            // 根据 查询
            if (ObjectUtil.isNotEmpty(eartagRegisterParam.getRegisterDate())) {
                queryWrapper.lambda().eq(EartagRegister::getRegisterDate, eartagRegisterParam.getRegisterDate());
            }
            // 根据 查询
            if (ObjectUtil.isNotEmpty(eartagRegisterParam.getAge())) {
                queryWrapper.lambda().eq(EartagRegister::getAge, eartagRegisterParam.getAge());
            }
            // 根据 查询
            if (ObjectUtil.isNotEmpty(eartagRegisterParam.getDeviceCode())) {
                queryWrapper.lambda().eq(EartagRegister::getDeviceCode, eartagRegisterParam.getDeviceCode());
            }
            // 根据 查询
            if (ObjectUtil.isNotEmpty(eartagRegisterParam.getFarmCode())) {
                queryWrapper.lambda().eq(EartagRegister::getFarmCode, eartagRegisterParam.getFarmCode());
            }
            // 根据 查询
            if (ObjectUtil.isNotEmpty(eartagRegisterParam.getStage())) {
                queryWrapper.lambda().eq(EartagRegister::getStage, eartagRegisterParam.getStage());
            }
            // 根据 查询
            if (ObjectUtil.isNotEmpty(eartagRegisterParam.getPigpen())) {
                queryWrapper.lambda().eq(EartagRegister::getPigpen, eartagRegisterParam.getPigpen());
            }
            // 根据 查询
            if (ObjectUtil.isNotEmpty(eartagRegisterParam.getWaertagAge())) {
                queryWrapper.lambda().eq(EartagRegister::getWaertagAge, eartagRegisterParam.getWaertagAge());
            }
        }
        return new PageResult<>(this.page(PageFactory.defaultPage(), queryWrapper));
    }

    @Override
    public List<EartagRegister> list(EartagRegisterParam eartagRegisterParam) {
        return this.list();
    }

    @Override
    public void add(EartagRegisterParam eartagRegisterParam) {
        EartagRegister eartagRegister = new EartagRegister();
        BeanUtil.copyProperties(eartagRegisterParam, eartagRegister);
        this.save(eartagRegister);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<EartagRegisterParam> eartagRegisterParamList) {
        eartagRegisterParamList.forEach(eartagRegisterParam -> {
            this.removeById(eartagRegisterParam.getId());
        });
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(EartagRegisterParam eartagRegisterParam) {
        EartagRegister eartagRegister = this.queryEartagRegister(eartagRegisterParam);
        BeanUtil.copyProperties(eartagRegisterParam, eartagRegister);
        this.updateById(eartagRegister);
    }

    @Override
    public EartagRegister detail(EartagRegisterParam eartagRegisterParam) {
        return this.queryEartagRegister(eartagRegisterParam);
    }

    /**
     * 获取耳标注册
     *
     * @author 杨迪
     * @date 2021-08-21 17:19:57
     */
    private EartagRegister queryEartagRegister(EartagRegisterParam eartagRegisterParam) {
        EartagRegister eartagRegister = this.getById(eartagRegisterParam.getId());
        if (ObjectUtil.isNull(eartagRegister)) {
            throw new ServiceException(EartagRegisterExceptionEnum.NOT_EXIST);
        }
        return eartagRegister;
    }

    @Override
    public void export(EartagRegisterParam eartagRegisterParam) {
        List<EartagRegister> list = this.list(eartagRegisterParam);
        PoiUtil.exportExcelWithStream("SnowyEartagRegister.xls", EartagRegister.class, list);
    }

}
