/*
Copyright [2020] [https://www.xiaonuo.vip]

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：

1.请不要删除和修改根目录下的LICENSE文件。
2.请不要删除和修改Snowy源码头部的版权声明。
3.请保留源码和相关描述文件的项目出处，作者声明等。
4.分发源码时候，请注明软件出处 https://gitee.com/xiaonuobase/snowy
5.在修改包名，模块名称，项目代码等时，请注明软件出处 https://gitee.com/xiaonuobase/snowy
6.若您的项目无法满足以上几点，可申请商业授权，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.huimv.modular.farmalarmnumber.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.core.consts.CommonConstant;
import com.huimv.core.enums.CommonStatusEnum;
import com.huimv.core.exception.ServiceException;
import com.huimv.core.factory.PageFactory;
import com.huimv.core.pojo.page.PageResult;
import com.huimv.core.util.PoiUtil;
import com.huimv.modular.farmalarmnumber.entity.FarmAlarmNumber;
import com.huimv.modular.farmalarmnumber.enums.FarmAlarmNumberExceptionEnum;
import com.huimv.modular.farmalarmnumber.mapper.FarmAlarmNumberMapper;
import com.huimv.modular.farmalarmnumber.param.FarmAlarmNumberParam;
import com.huimv.modular.farmalarmnumber.service.FarmAlarmNumberService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import javax.annotation.Resource;
import java.util.List;

/**
 * alarm_numberservice接口实现类
 *
 * @author yangdi
 * @date 2021-08-23 15:12:46
 */
@Service
public class FarmAlarmNumberServiceImpl extends ServiceImpl<FarmAlarmNumberMapper, FarmAlarmNumber> implements FarmAlarmNumberService {

    @Override
    public PageResult<FarmAlarmNumber> page(FarmAlarmNumberParam farmAlarmNumberParam) {
        QueryWrapper<FarmAlarmNumber> queryWrapper = new QueryWrapper<>();
        if (ObjectUtil.isNotNull(farmAlarmNumberParam)) {

            // 根据 查询
            if (ObjectUtil.isNotEmpty(farmAlarmNumberParam.getFarmCode())) {
                queryWrapper.lambda().eq(FarmAlarmNumber::getFarmCode, farmAlarmNumberParam.getFarmCode());
            }
            // 根据 查询
            if (ObjectUtil.isNotEmpty(farmAlarmNumberParam.getFarmName())) {
                queryWrapper.lambda().eq(FarmAlarmNumber::getFarmName, farmAlarmNumberParam.getFarmName());
            }
            // 根据组织ID 查询
            if (ObjectUtil.isNotEmpty(farmAlarmNumberParam.getOrgid())) {
                queryWrapper.lambda().eq(FarmAlarmNumber::getOrgid, farmAlarmNumberParam.getOrgid());
            }
            // 根据 查询
            if (ObjectUtil.isNotEmpty(farmAlarmNumberParam.getEarHightemp())) {
                queryWrapper.lambda().eq(FarmAlarmNumber::getEarHightemp, farmAlarmNumberParam.getEarHightemp());
            }
            // 根据 查询
            if (ObjectUtil.isNotEmpty(farmAlarmNumberParam.getEarLowtemp())) {
                queryWrapper.lambda().eq(FarmAlarmNumber::getEarLowtemp, farmAlarmNumberParam.getEarLowtemp());
            }
            // 根据 查询
            if (ObjectUtil.isNotEmpty(farmAlarmNumberParam.getEnvHightemp())) {
                queryWrapper.lambda().eq(FarmAlarmNumber::getEnvHightemp, farmAlarmNumberParam.getEnvHightemp());
            }
            // 根据 查询
            if (ObjectUtil.isNotEmpty(farmAlarmNumberParam.getEnvLowtemp())) {
                queryWrapper.lambda().eq(FarmAlarmNumber::getEnvLowtemp, farmAlarmNumberParam.getEnvLowtemp());
            }
            // 根据 查询
            if (ObjectUtil.isNotEmpty(farmAlarmNumberParam.getDaviceAlarm())) {
                queryWrapper.lambda().eq(FarmAlarmNumber::getDaviceAlarm, farmAlarmNumberParam.getDaviceAlarm());
            }
            // 根据 查询
            if (ObjectUtil.isNotEmpty(farmAlarmNumberParam.getDiseaseAlarm())) {
                queryWrapper.lambda().eq(FarmAlarmNumber::getDiseaseAlarm, farmAlarmNumberParam.getDiseaseAlarm());
            }
            // 根据 查询
            if (ObjectUtil.isNotEmpty(farmAlarmNumberParam.getRemark())) {
                queryWrapper.lambda().eq(FarmAlarmNumber::getRemark, farmAlarmNumberParam.getRemark());
            }
            // 根据 查询
            if (ObjectUtil.isNotEmpty(farmAlarmNumberParam.getRemark1())) {
                queryWrapper.lambda().eq(FarmAlarmNumber::getRemark1, farmAlarmNumberParam.getRemark1());
            }
        }
        return new PageResult<>(this.page(PageFactory.defaultPage(), queryWrapper));
    }

    @Override
    public List<FarmAlarmNumber> list(FarmAlarmNumberParam farmAlarmNumberParam) {
        return this.list();
    }

    @Override
    public void add(FarmAlarmNumberParam farmAlarmNumberParam) {
        FarmAlarmNumber farmAlarmNumber = new FarmAlarmNumber();
        BeanUtil.copyProperties(farmAlarmNumberParam, farmAlarmNumber);
        this.save(farmAlarmNumber);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<FarmAlarmNumberParam> farmAlarmNumberParamList) {
        farmAlarmNumberParamList.forEach(farmAlarmNumberParam -> {
            this.removeById(farmAlarmNumberParam.getId());
        });
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(FarmAlarmNumberParam farmAlarmNumberParam) {
        FarmAlarmNumber farmAlarmNumber = this.queryFarmAlarmNumber(farmAlarmNumberParam);
        BeanUtil.copyProperties(farmAlarmNumberParam, farmAlarmNumber);
        this.updateById(farmAlarmNumber);
    }

    @Override
    public FarmAlarmNumber detail(FarmAlarmNumberParam farmAlarmNumberParam) {
        return this.queryFarmAlarmNumber(farmAlarmNumberParam);
    }

    /**
     * 获取alarm_number
     *
     * @author yangdi
     * @date 2021-08-23 15:12:46
     */
    private FarmAlarmNumber queryFarmAlarmNumber(FarmAlarmNumberParam farmAlarmNumberParam) {
        FarmAlarmNumber farmAlarmNumber = this.getById(farmAlarmNumberParam.getId());
        if (ObjectUtil.isNull(farmAlarmNumber)) {
            throw new ServiceException(FarmAlarmNumberExceptionEnum.NOT_EXIST);
        }
        return farmAlarmNumber;
    }

    @Override
    public void export(FarmAlarmNumberParam farmAlarmNumberParam) {
        List<FarmAlarmNumber> list = this.list(farmAlarmNumberParam);
        PoiUtil.exportExcelWithStream("SnowyFarmAlarmNumber.xls", FarmAlarmNumber.class, list);
    }

}
