/*
Copyright [2020] [https://www.xiaonuo.vip]

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：

1.请不要删除和修改根目录下的LICENSE文件。
2.请不要删除和修改Snowy源码头部的版权声明。
3.请保留源码和相关描述文件的项目出处，作者声明等。
4.分发源码时候，请注明软件出处 https://gitee.com/xiaonuobase/snowy
5.在修改包名，模块名称，项目代码等时，请注明软件出处 https://gitee.com/xiaonuobase/snowy
6.若您的项目无法满足以上几点，可申请商业授权，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.huimv.modular.stockout.controller;

import com.huimv.core.annotion.BusinessLog;
import com.huimv.core.annotion.Permission;
import com.huimv.core.enums.LogAnnotionOpTypeEnum;
import com.huimv.core.pojo.response.ResponseData;
import com.huimv.core.pojo.response.SuccessResponseData;
import com.huimv.modular.stockout.param.StockOutParam;
import com.huimv.modular.stockout.service.StockOutService;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import javax.annotation.Resource;
import java.util.List;

/**
 * 累计出栏控制器
 *
 * @author y
 * @date 2021-08-21 14:28:14
 */
@RestController
public class StockOutController {

    @Resource
    private StockOutService stockOutService;

    /**
     * 查询累计出栏
     *
     * @author y
     * @date 2021-08-21 14:28:14
     */

    @GetMapping("/stockOut/page")
    @BusinessLog(title = "累计出栏_查询", opType = LogAnnotionOpTypeEnum.QUERY)
    public ResponseData page(StockOutParam stockOutParam) {
        return new SuccessResponseData(stockOutService.page(stockOutParam));
    }

    /**
     * 添加累计出栏
     *
     * @author y
     * @date 2021-08-21 14:28:14
     */

    @PostMapping("/stockOut/add")
    @BusinessLog(title = "累计出栏_增加", opType = LogAnnotionOpTypeEnum.ADD)
    public ResponseData add(@RequestBody @Validated(StockOutParam.add.class) StockOutParam stockOutParam) {
            stockOutService.add(stockOutParam);
        return new SuccessResponseData();
    }

    /**
     * 删除累计出栏，可批量删除
     *
     * @author y
     * @date 2021-08-21 14:28:14
     */

    @PostMapping("/stockOut/delete")
    @BusinessLog(title = "累计出栏_删除", opType = LogAnnotionOpTypeEnum.DELETE)
    public ResponseData delete(@RequestBody @Validated(StockOutParam.delete.class) List<StockOutParam> stockOutParamList) {
            stockOutService.delete(stockOutParamList);
        return new SuccessResponseData();
    }

    /**
     * 编辑累计出栏
     *
     * @author y
     * @date 2021-08-21 14:28:14
     */

    @PostMapping("/stockOut/edit")
    @BusinessLog(title = "累计出栏_编辑", opType = LogAnnotionOpTypeEnum.EDIT)
    public ResponseData edit(@RequestBody @Validated(StockOutParam.edit.class) StockOutParam stockOutParam) {
            stockOutService.edit(stockOutParam);
        return new SuccessResponseData();
    }

    /**
     * 查看累计出栏
     *
     * @author y
     * @date 2021-08-21 14:28:14
     */

    @GetMapping("/stockOut/detail")
    @BusinessLog(title = "累计出栏_查看", opType = LogAnnotionOpTypeEnum.DETAIL)
    public ResponseData detail(@Validated(StockOutParam.detail.class) StockOutParam stockOutParam) {
        return new SuccessResponseData(stockOutService.detail(stockOutParam));
    }

    /**
     * 累计出栏列表
     *
     * @author y
     * @date 2021-08-21 14:28:14
     */

    @GetMapping("/stockOut/list")
    @BusinessLog(title = "累计出栏_列表", opType = LogAnnotionOpTypeEnum.QUERY)
    public ResponseData list(StockOutParam stockOutParam) {
        return new SuccessResponseData(stockOutService.list(stockOutParam));
    }

    /**
     * 导出系统用户
     *
     * @author y
     * @date 2021-08-21 14:28:14
     */

    @GetMapping("/stockOut/export")
    @BusinessLog(title = "累计出栏_导出", opType = LogAnnotionOpTypeEnum.EXPORT)
    public void export(StockOutParam stockOutParam) {
        stockOutService.export(stockOutParam);
    }

}
