/*
Copyright [2020] [https://www.xiaonuo.vip]

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：

1.请不要删除和修改根目录下的LICENSE文件。
2.请不要删除和修改Snowy源码头部的版权声明。
3.请保留源码和相关描述文件的项目出处，作者声明等。
4.分发源码时候，请注明软件出处 https://gitee.com/xiaonuobase/snowy
5.在修改包名，模块名称，项目代码等时，请注明软件出处 https://gitee.com/xiaonuobase/snowy
6.若您的项目无法满足以上几点，可申请商业授权，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.huimv.modular.stockout.param;

import com.huimv.core.pojo.base.param.BaseParam;
import lombok.Data;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.NotBlank;
import java.util.*;

/**
* 累计出栏参数类
 *
 * @author y
 * @date 2021-08-21 14:28:14
*/
@Data
public class StockOutParam extends BaseParam {

    /**
     * 
     */
    @NotNull(message = "不能为空，请检查id参数", groups = {edit.class, delete.class, detail.class})
    private Integer id;

    /**
     * 行政区划代码/牧场代码
     */
    @NotNull(message = "行政区划代码/牧场代码不能为空，请检查orgId参数", groups = {add.class, edit.class})
    private Long orgId;

    /**
     * 行政区划名称/牧场名称（备用字段）
     */
    @NotBlank(message = "行政区划名称/牧场名称（备用字段）不能为空，请检查orgName参数", groups = {add.class, edit.class})
    private String orgName;

    /**
     * 母猪总存栏
     */
    @NotNull(message = "母猪总存栏不能为空，请检查sowTotalStock参数", groups = {add.class, edit.class})
    private Integer sowTotalStock;

    /**
     * 总存栏增幅类型
     */
    @NotNull(message = "总存栏增幅类型不能为空，请检查totalStockWay参数", groups = {add.class, edit.class})
    private Integer totalStockWay;

    /**
     * 总存栏增幅数量
     */
    @NotNull(message = "总存栏增幅数量不能为空，请检查totalStockUpdate参数", groups = {add.class, edit.class})
    private Integer totalStockUpdate;

    /**
     * 累计出栏量
     */
    @NotNull(message = "累计出栏量不能为空，请检查outTotal参数", groups = {add.class, edit.class})
    private Integer outTotal;

    /**
     * 累计出栏量增幅类型
     */
    @NotNull(message = "累计出栏量增幅类型不能为空，请检查outWay参数", groups = {add.class, edit.class})
    private Integer outWay;

    /**
     * 累计出栏量增幅数量
     */
    @NotNull(message = "累计出栏量增幅数量不能为空，请检查outUpdate参数", groups = {add.class, edit.class})
    private Integer outUpdate;

    /**
     * 预计年出栏量
     */
    @NotNull(message = "预计年出栏量不能为空，请检查expectOut参数", groups = {add.class, edit.class})
    private Integer expectOut;

    /**
     * 预计年出栏量增幅类型
     */
    @NotNull(message = "预计年出栏量增幅类型不能为空，请检查expectOutWay参数", groups = {add.class, edit.class})
    private Integer expectOutWay;

    /**
     * 预计年出栏量增幅数量
     */
    @NotNull(message = "预计年出栏量增幅数量不能为空，请检查expectOutUpdate参数", groups = {add.class, edit.class})
    private Integer expectOutUpdate;

    /**
     * 母猪目标存栏量
     */
    @NotNull(message = "母猪目标存栏量不能为空，请检查sowPlanStock参数", groups = {add.class, edit.class})
    private Integer sowPlanStock;

    /**
     * 目标年出栏量
     */
    @NotNull(message = "目标年出栏量不能为空，请检查planOut参数", groups = {add.class, edit.class})
    private Integer planOut;

    /**
     * 年度
     */
    @NotNull(message = "年度不能为空，请检查year参数", groups = {add.class, edit.class})
    private Integer year;

}
