/*
Copyright [2020] [https://www.xiaonuo.vip]

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：

1.请不要删除和修改根目录下的LICENSE文件。
2.请不要删除和修改Snowy源码头部的版权声明。
3.请保留源码和相关描述文件的项目出处，作者声明等。
4.分发源码时候，请注明软件出处 https://gitee.com/xiaonuobase/snowy
5.在修改包名，模块名称，项目代码等时，请注明软件出处 https://gitee.com/xiaonuobase/snowy
6.若您的项目无法满足以上几点，可申请商业授权，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.huimv.modular.stockout.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.core.consts.CommonConstant;
import com.huimv.core.enums.CommonStatusEnum;
import com.huimv.core.exception.ServiceException;
import com.huimv.core.factory.PageFactory;
import com.huimv.core.pojo.page.PageResult;
import com.huimv.core.util.PoiUtil;
import com.huimv.modular.stockout.entity.StockOut;
import com.huimv.modular.stockout.enums.StockOutExceptionEnum;
import com.huimv.modular.stockout.mapper.StockOutMapper;
import com.huimv.modular.stockout.param.StockOutParam;
import com.huimv.modular.stockout.service.StockOutService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import javax.annotation.Resource;
import java.util.List;

/**
 * 累计出栏service接口实现类
 *
 * @author y
 * @date 2021-08-21 14:28:14
 */
@Service
public class StockOutServiceImpl extends ServiceImpl<StockOutMapper, StockOut> implements StockOutService {

    @Override
    public PageResult<StockOut> page(StockOutParam stockOutParam) {
        QueryWrapper<StockOut> queryWrapper = new QueryWrapper<>();
        if (ObjectUtil.isNotNull(stockOutParam)) {

            // 根据行政区划代码/牧场代码 查询
            if (ObjectUtil.isNotEmpty(stockOutParam.getOrgId())) {
                queryWrapper.lambda().eq(StockOut::getOrgId, stockOutParam.getOrgId());
            }
            // 根据行政区划名称/牧场名称（备用字段） 查询
            if (ObjectUtil.isNotEmpty(stockOutParam.getOrgName())) {
                queryWrapper.lambda().eq(StockOut::getOrgName, stockOutParam.getOrgName());
            }
            // 根据母猪总存栏 查询
            if (ObjectUtil.isNotEmpty(stockOutParam.getSowTotalStock())) {
                queryWrapper.lambda().eq(StockOut::getSowTotalStock, stockOutParam.getSowTotalStock());
            }
            // 根据总存栏增幅类型 查询
            if (ObjectUtil.isNotEmpty(stockOutParam.getTotalStockWay())) {
                queryWrapper.lambda().eq(StockOut::getTotalStockWay, stockOutParam.getTotalStockWay());
            }
            // 根据总存栏增幅数量 查询
            if (ObjectUtil.isNotEmpty(stockOutParam.getTotalStockUpdate())) {
                queryWrapper.lambda().eq(StockOut::getTotalStockUpdate, stockOutParam.getTotalStockUpdate());
            }
            // 根据累计出栏量 查询
            if (ObjectUtil.isNotEmpty(stockOutParam.getOutTotal())) {
                queryWrapper.lambda().eq(StockOut::getOutTotal, stockOutParam.getOutTotal());
            }
            // 根据累计出栏量增幅类型 查询
            if (ObjectUtil.isNotEmpty(stockOutParam.getOutWay())) {
                queryWrapper.lambda().eq(StockOut::getOutWay, stockOutParam.getOutWay());
            }
            // 根据累计出栏量增幅数量 查询
            if (ObjectUtil.isNotEmpty(stockOutParam.getOutUpdate())) {
                queryWrapper.lambda().eq(StockOut::getOutUpdate, stockOutParam.getOutUpdate());
            }
            // 根据预计年出栏量 查询
            if (ObjectUtil.isNotEmpty(stockOutParam.getExpectOut())) {
                queryWrapper.lambda().eq(StockOut::getExpectOut, stockOutParam.getExpectOut());
            }
            // 根据预计年出栏量增幅类型 查询
            if (ObjectUtil.isNotEmpty(stockOutParam.getExpectOutWay())) {
                queryWrapper.lambda().eq(StockOut::getExpectOutWay, stockOutParam.getExpectOutWay());
            }
            // 根据预计年出栏量增幅数量 查询
            if (ObjectUtil.isNotEmpty(stockOutParam.getExpectOutUpdate())) {
                queryWrapper.lambda().eq(StockOut::getExpectOutUpdate, stockOutParam.getExpectOutUpdate());
            }
            // 根据母猪目标存栏量 查询
            if (ObjectUtil.isNotEmpty(stockOutParam.getSowPlanStock())) {
                queryWrapper.lambda().eq(StockOut::getSowPlanStock, stockOutParam.getSowPlanStock());
            }
            // 根据目标年出栏量 查询
            if (ObjectUtil.isNotEmpty(stockOutParam.getPlanOut())) {
                queryWrapper.lambda().eq(StockOut::getPlanOut, stockOutParam.getPlanOut());
            }
            // 根据年度 查询
            if (ObjectUtil.isNotEmpty(stockOutParam.getYear())) {
                queryWrapper.lambda().eq(StockOut::getYear, stockOutParam.getYear());
            }
        }
        return new PageResult<>(this.page(PageFactory.defaultPage(), queryWrapper));
    }

    @Override
    public List<StockOut> list(StockOutParam stockOutParam) {
        return this.list();
    }

    @Override
    public void add(StockOutParam stockOutParam) {
        StockOut stockOut = new StockOut();
        BeanUtil.copyProperties(stockOutParam, stockOut);
        this.save(stockOut);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<StockOutParam> stockOutParamList) {
        stockOutParamList.forEach(stockOutParam -> {
            this.removeById(stockOutParam.getId());
        });
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(StockOutParam stockOutParam) {
        StockOut stockOut = this.queryStockOut(stockOutParam);
        BeanUtil.copyProperties(stockOutParam, stockOut);
        this.updateById(stockOut);
    }

    @Override
    public StockOut detail(StockOutParam stockOutParam) {
        return this.queryStockOut(stockOutParam);
    }

    /**
     * 获取累计出栏
     *
     * @author y
     * @date 2021-08-21 14:28:14
     */
    private StockOut queryStockOut(StockOutParam stockOutParam) {
        StockOut stockOut = this.getById(stockOutParam.getId());
        if (ObjectUtil.isNull(stockOut)) {
            throw new ServiceException(StockOutExceptionEnum.NOT_EXIST);
        }
        return stockOut;
    }

    @Override
    public void export(StockOutParam stockOutParam) {
        List<StockOut> list = this.list(stockOutParam);
        PoiUtil.exportExcelWithStream("SnowyStockOut.xls", StockOut.class, list);
    }

}
