/*
Copyright [2020] [https://www.xiaonuo.vip]

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：

1.请不要删除和修改根目录下的LICENSE文件。
2.请不要删除和修改Snowy源码头部的版权声明。
3.请保留源码和相关描述文件的项目出处，作者声明等。
4.分发源码时候，请注明软件出处 https://gitee.com/xiaonuobase/snowy
5.在修改包名，模块名称，项目代码等时，请注明软件出处 https://gitee.com/xiaonuobase/snowy
6.若您的项目无法满足以上几点，可申请商业授权，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.huimv.modular.stockoutmonth.controller;

import com.huimv.core.annotion.BusinessLog;
import com.huimv.core.annotion.Permission;
import com.huimv.core.enums.LogAnnotionOpTypeEnum;
import com.huimv.core.pojo.response.ResponseData;
import com.huimv.core.pojo.response.SuccessResponseData;
import com.huimv.modular.stockoutmonth.param.StockOutMonthParam;
import com.huimv.modular.stockoutmonth.service.StockOutMonthService;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * 出栏统计，月度控制器
 *
 * @author y
 * @date 2021-08-21 14:28:17
 */
@RestController
public class StockOutMonthController {

    @Resource
    private StockOutMonthService stockOutMonthService;

    /**
     * 查询出栏统计，月度
     *
     * @author y
     * @date 2021-08-21 14:28:17
     */

    @GetMapping("/stockOutMonth/page")
    public ResponseData page(StockOutMonthParam stockOutMonthParam) {
        return new SuccessResponseData(stockOutMonthService.page(stockOutMonthParam));
    }

    /**
     * 添加出栏统计，月度
     *
     * @author y
     * @date 2021-08-21 14:28:17
     */

    @PostMapping("/stockOutMonth/add")
    public ResponseData add(@RequestBody @Validated(StockOutMonthParam.add.class) StockOutMonthParam stockOutMonthParam) {

        return stockOutMonthService.add(stockOutMonthParam);
    }

    /**
     * 删除出栏统计，月度，可批量删除
     *
     * @author y
     * @date 2021-08-21 14:28:17
     */

    @PostMapping("/stockOutMonth/delete")
    public ResponseData delete(@RequestBody List<Integer> ids) {
            stockOutMonthService.delete(ids);
        return new SuccessResponseData();
    }

    /**
     * 编辑出栏统计，月度
     *
     * @author y
     * @date 2021-08-21 14:28:17
     */

    @PostMapping("/stockOutMonth/edit")
    public ResponseData edit(@RequestBody @Validated(StockOutMonthParam.edit.class) StockOutMonthParam stockOutMonthParam) {
            stockOutMonthService.edit(stockOutMonthParam);
        return new SuccessResponseData();
    }

    /**
     * 查看出栏统计，月度
     *
     * @author y
     * @date 2021-08-21 14:28:17
     */

    @GetMapping("/stockOutMonth/detail")
    public ResponseData detail(@Validated(StockOutMonthParam.detail.class) StockOutMonthParam stockOutMonthParam) {
        return new SuccessResponseData(stockOutMonthService.detail(stockOutMonthParam));
    }

    /**
     * 出栏统计，月度列表
     *
     * @author y
     * @date 2021-08-21 14:28:17
     */

    @GetMapping("/stockOutMonth/list")
    public ResponseData list(StockOutMonthParam stockOutMonthParam) {
        return new SuccessResponseData(stockOutMonthService.list(stockOutMonthParam));
    }

    /**
     * 导出系统用户
     *
     * @author y
     * @date 2021-08-21 14:28:17
     */

    @GetMapping("/stockOutMonth/export")
    public void export(StockOutMonthParam stockOutMonthParam) {
        stockOutMonthService.export(stockOutMonthParam);
    }


    @GetMapping("/stockOutMonth/outWarning")
    public ResponseData outWarning() {
        Map<String,Object> map = stockOutMonthService.outWarning();
        return new SuccessResponseData(map);
    }

}
