/*
Copyright [2020] [https://www.xiaonuo.vip]

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：

1.请不要删除和修改根目录下的LICENSE文件。
2.请不要删除和修改Snowy源码头部的版权声明。
3.请保留源码和相关描述文件的项目出处，作者声明等。
4.分发源码时候，请注明软件出处 https://gitee.com/xiaonuobase/snowy
5.在修改包名，模块名称，项目代码等时，请注明软件出处 https://gitee.com/xiaonuobase/snowy
6.若您的项目无法满足以上几点，可申请商业授权，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.huimv.modular.stockoutmonth.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.core.consts.CommonConstant;
import com.huimv.core.context.login.LoginContext;
import com.huimv.core.context.login.LoginContextHolder;
import com.huimv.core.enums.CommonStatusEnum;
import com.huimv.core.exception.ServiceException;
import com.huimv.core.factory.PageFactory;
import com.huimv.core.pojo.page.PageResult;
import com.huimv.core.pojo.response.ErrorResponseData;
import com.huimv.core.pojo.response.ResponseData;
import com.huimv.core.pojo.response.SuccessResponseData;
import com.huimv.core.util.PoiUtil;
import com.huimv.modular.stockoutmonth.entity.StockOutMonth;
import com.huimv.modular.stockoutmonth.enums.StockOutMonthExceptionEnum;
import com.huimv.modular.stockoutmonth.mapper.StockOutMonthMapper;
import com.huimv.modular.stockoutmonth.param.StockOutMonthParam;
import com.huimv.modular.stockoutmonth.service.StockOutMonthService;
import com.huimv.sys.modular.org.entity.SysOrg;
import com.huimv.sys.modular.org.mapper.SysOrgMapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import javax.annotation.Resource;
import java.util.*;

/**
 * 出栏统计，月度service接口实现类
 *
 * @author y
 * @date 2021-08-21 14:28:17
 */
@Service
public class StockOutMonthServiceImpl extends ServiceImpl<StockOutMonthMapper, StockOutMonth> implements StockOutMonthService {

    @Resource
    private SysOrgMapper sysOrgMapper;
    @Override
    public PageResult<StockOutMonth> page(StockOutMonthParam stockOutMonthParam) {
        QueryWrapper<StockOutMonth> queryWrapper = new QueryWrapper<>();
        Long orgId = null;
        if (ObjectUtil.isNotNull(stockOutMonthParam)) {
            orgId   = stockOutMonthParam.getOrgId();
            if (ObjectUtil.isNotNull(stockOutMonthParam.getYear())){
                queryWrapper.in("year",stockOutMonthParam.getYear());
            }
        }
        if (ObjectUtil.isEmpty(orgId)){
            orgId = LoginContextHolder.me().getSysLoginUserOrgId();
        }

        List<SysOrg> pids = sysOrgMapper.selectList(new QueryWrapper<SysOrg>().like("pids", orgId));
        List<Long> orgPids = new ArrayList<>();
        for (SysOrg pid : pids) {
            orgPids.add(pid.getId());
        }
        orgPids.add(orgId);
        queryWrapper.eq("stock_type",1);
        queryWrapper.in("org_id",orgPids);
        queryWrapper.orderByDesc("id");

        return new PageResult<>(this.page(PageFactory.defaultPage(), queryWrapper));
    }

    @Override
    public List<StockOutMonth> list(StockOutMonthParam stockOutMonthParam) {
        return this.list();
    }

    @Override
    public ResponseData add(StockOutMonthParam stockOutMonthParam) {

        StockOutMonth stockOutMonth = new StockOutMonth();
        BeanUtil.copyProperties(stockOutMonthParam, stockOutMonth);

        Long orgId = LoginContextHolder.me().getSysLoginUserOrgId();


        QueryWrapper<StockOutMonth> wrapper = new QueryWrapper<>();
        wrapper.eq("org_id",orgId);
        wrapper.eq("year", DateUtil.year(new Date()));
        wrapper.eq("stock_type", stockOutMonth.getStockType());
        List<StockOutMonth> list = this.list(wrapper);
        if (list.size()>0){
            return new ErrorResponseData("该条信息已存在，请修改");
        }

        SysOrg sysOrg = sysOrgMapper.selectById(orgId);

        stockOutMonth.setOrgId(orgId);
        stockOutMonth.setOrgName(sysOrg.getName());
        stockOutMonth.setYear(DateUtil.year(new Date()));
        this.save(stockOutMonth);
        addPids(orgId,stockOutMonth);
        return new SuccessResponseData();
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<Integer> ids) {
        for (Integer id : ids) {
            StockOutMonth byId = this.getById(id);
            subtract(byId.getOrgId(),byId);
        }

        ids.forEach(this::removeById);
    }



    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(StockOutMonthParam stockOutMonthParam) {
        StockOutMonth stockOutMonth = this.queryStockOutMonth(stockOutMonthParam);
        Long orgId = LoginContextHolder.me().getSysLoginUserOrgId();
        subtract(orgId,stockOutMonth);
        BeanUtil.copyProperties(stockOutMonthParam, stockOutMonth);
        addPids(orgId,stockOutMonth);

        this.updateById(stockOutMonth);
    }

    @Override
    public StockOutMonth detail(StockOutMonthParam stockOutMonthParam) {
        return this.queryStockOutMonth(stockOutMonthParam);
    }

    /**
     * 获取出栏统计，月度
     *
     * @author y
     * @date 2021-08-21 14:28:17
     */
    private StockOutMonth queryStockOutMonth(StockOutMonthParam stockOutMonthParam) {
        StockOutMonth stockOutMonth = this.getById(stockOutMonthParam.getId());
        if (ObjectUtil.isNull(stockOutMonth)) {
            throw new ServiceException(StockOutMonthExceptionEnum.NOT_EXIST);
        }
        return stockOutMonth;
    }

    @Override
    public void export(StockOutMonthParam stockOutMonthParam) {
        List<StockOutMonth> list = this.list(stockOutMonthParam);
        PoiUtil.exportExcelWithStream("SnowyStockOutMonth.xls", StockOutMonth.class, list);
    }

    @Override
    public Map<String, Object> outWarning() {
        Map<String,Object> endMap = new HashMap<>();
        Long orgId = LoginContextHolder.me().getSysLoginUserOrgId();

        QueryWrapper<StockOutMonth> wrapper = new QueryWrapper<>();
        wrapper.eq("org_id",orgId);
        wrapper.eq("year", DateUtil.year(new Date()));
        List<StockOutMonth> list = this.list(wrapper);
        //今年累计出栏
        Integer realStockOutCount;
        //今年预计出栏
        Integer predictStockOut  =0;
        for (StockOutMonth stockOutMonth : list) {
            if (stockOutMonth.getStockType() == 1){
                realStockOutCount = sumYear(stockOutMonth);
                //今年目标存栏
                Integer targetStockOutCount =  stockOutMonth.getTargetStockOut();

                endMap.put("realStockOut",stockOutMonth);
                endMap.put("realStockOutCount",realStockOutCount);
                endMap.put("targetStockOutCount",targetStockOutCount);
                endMap.put("realPercentage",(realStockOutCount*100)/targetStockOutCount);
            }
            if (stockOutMonth.getStockType() == 2){
                predictStockOut = sumYear(stockOutMonth);
                endMap.put("predictStockOut",stockOutMonth);
                endMap.put("predictStockOutCount",predictStockOut);
            }
        }
        endMap.put("predictPercentage",predictStockOut);


        return endMap;
    }

    private Integer sumYear(StockOutMonth stockOutMonth) {
       Integer count = 0;

       if (ObjectUtil.isNotEmpty(stockOutMonth.getMonth1())){
           count += stockOutMonth.getMonth1();
       }
        if (ObjectUtil.isNotEmpty(stockOutMonth.getMonth2())){
            count += stockOutMonth.getMonth2();
        }
        if (ObjectUtil.isNotEmpty(stockOutMonth.getMonth3())){
            count += stockOutMonth.getMonth3();
        }
        if (ObjectUtil.isNotEmpty(stockOutMonth.getMonth4())){
            count += stockOutMonth.getMonth4();
        }
        if (ObjectUtil.isNotEmpty(stockOutMonth.getMonth5())){
            count += stockOutMonth.getMonth5();
        }
        if (ObjectUtil.isNotEmpty(stockOutMonth.getMonth6())){
            count += stockOutMonth.getMonth6();
        }
        if (ObjectUtil.isNotEmpty(stockOutMonth.getMonth7())){
            count += stockOutMonth.getMonth7();
        }
        if (ObjectUtil.isNotEmpty(stockOutMonth.getMonth8())){
            count += stockOutMonth.getMonth8();
        }
        if (ObjectUtil.isNotEmpty(stockOutMonth.getMonth9())){
            count += stockOutMonth.getMonth9();
        }
        if (ObjectUtil.isNotEmpty(stockOutMonth.getMonth10())){
            count += stockOutMonth.getMonth10();
        }
        if (ObjectUtil.isNotEmpty(stockOutMonth.getMonth11())){
            count += stockOutMonth.getMonth11();
        }
        if (ObjectUtil.isNotEmpty(stockOutMonth.getMonth12())){
            count += stockOutMonth.getMonth12();
        }


       return count;

    }

    //添加父级
    private void addPids(Long orgId,StockOutMonth stockOutMonthParam) {
        SysOrg sysOrg = sysOrgMapper.selectById(orgId);
        SysOrg sysOrgPid = sysOrgMapper.selectById(sysOrg.getPid());
        Long orgIdPid = sysOrgPid.getId();
        QueryWrapper<StockOutMonth> wrapper = new QueryWrapper<>();
        wrapper.eq("org_id",orgIdPid);
        wrapper.eq("year", DateUtil.year(new Date()));
        wrapper.eq("stock_type",1);
        List<StockOutMonth> list = this.list(wrapper);
        if (list.size() > 0){
            StockOutMonth stockOutMonth = list.get(0);
            stockOutMonth.setMonth1(stockOutMonth.getMonth1()+stockOutMonthParam.getMonth1());
            stockOutMonth.setMonth2(stockOutMonth.getMonth2()+stockOutMonthParam.getMonth2());
            stockOutMonth.setMonth3(stockOutMonth.getMonth3()+stockOutMonthParam.getMonth3());
            stockOutMonth.setMonth4(stockOutMonth.getMonth4()+stockOutMonthParam.getMonth4());
            stockOutMonth.setMonth5(stockOutMonth.getMonth5()+stockOutMonthParam.getMonth5());
            stockOutMonth.setMonth6(stockOutMonth.getMonth6()+stockOutMonthParam.getMonth6());
            stockOutMonth.setMonth7(stockOutMonth.getMonth7()+stockOutMonthParam.getMonth7());
            stockOutMonth.setMonth8(stockOutMonth.getMonth8()+stockOutMonthParam.getMonth8());
            stockOutMonth.setMonth9(stockOutMonth.getMonth9()+stockOutMonthParam.getMonth9());
            stockOutMonth.setMonth10(stockOutMonth.getMonth10()+stockOutMonthParam.getMonth10());
            stockOutMonth.setMonth11(stockOutMonth.getMonth11()+stockOutMonthParam.getMonth11());
            stockOutMonth.setMonth12(stockOutMonth.getMonth12()+stockOutMonthParam.getMonth12());
            stockOutMonth.setTargetStockOut(stockOutMonth.getTargetStockOut()+stockOutMonthParam.getTargetStockOut());
            baseMapper.updateById(stockOutMonth);
        }else {
            stockOutMonthParam.setOrgId(orgIdPid);
            stockOutMonthParam.setStockType(1);
            stockOutMonthParam.setOrgName(sysOrgPid.getName());
            stockOutMonthParam.setYear(DateUtil.year(new Date()));
            this.save(stockOutMonthParam);
        }
        if (sysOrgPid.getPid() != 0L){
            addPids(orgIdPid,stockOutMonthParam);
        }

    }

    private void subtract(Long orgId, StockOutMonth byId) {
        SysOrg sysOrg = sysOrgMapper.selectById(orgId);
        SysOrg sysOrgPid = sysOrgMapper.selectById(sysOrg.getPid());
        Long orgIdPid = sysOrgPid.getId();
        QueryWrapper<StockOutMonth> wrapper = new QueryWrapper<>();
        wrapper.eq("org_id",orgIdPid);
        wrapper.eq("year", DateUtil.year(new Date()));
        wrapper.eq("stock_type",1);
        List<StockOutMonth> list = this.list(wrapper);

        StockOutMonth stockOutMonth = list.get(0);
        stockOutMonth.setMonth1(stockOutMonth.getMonth1()-byId.getMonth1());
        stockOutMonth.setMonth2(stockOutMonth.getMonth2()-byId.getMonth2());
        stockOutMonth.setMonth3(stockOutMonth.getMonth3()-byId.getMonth3());
        stockOutMonth.setMonth4(stockOutMonth.getMonth4()-byId.getMonth4());
        stockOutMonth.setMonth5(stockOutMonth.getMonth5()-byId.getMonth5());
        stockOutMonth.setMonth6(stockOutMonth.getMonth6()-byId.getMonth6());
        stockOutMonth.setMonth7(stockOutMonth.getMonth7()-byId.getMonth7());
        stockOutMonth.setMonth8(stockOutMonth.getMonth8()-byId.getMonth8());
        stockOutMonth.setMonth9(stockOutMonth.getMonth9()-byId.getMonth9());
        stockOutMonth.setMonth10(stockOutMonth.getMonth10()-byId.getMonth10());
        stockOutMonth.setMonth11(stockOutMonth.getMonth11()-byId.getMonth11());
        stockOutMonth.setMonth12(stockOutMonth.getMonth12()-byId.getMonth12());
        stockOutMonth.setTargetStockOut(stockOutMonth.getTargetStockOut()-byId.getTargetStockOut());
        baseMapper.updateById(stockOutMonth);

        if (sysOrgPid.getPid() != 0L){
            subtract(orgIdPid,byId);
        }

    }

}
