/*
Copyright [2020] [https://www.xiaonuo.vip]

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：

1.请不要删除和修改根目录下的LICENSE文件。
2.请不要删除和修改Snowy源码头部的版权声明。
3.请保留源码和相关描述文件的项目出处，作者声明等。
4.分发源码时候，请注明软件出处 https://gitee.com/xiaonuobase/snowy
5.在修改包名，模块名称，项目代码等时，请注明软件出处 https://gitee.com/xiaonuobase/snowy
6.若您的项目无法满足以上几点，可申请商业授权，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.huimv.modular.stockplan.controller;

import com.huimv.core.annotion.BusinessLog;
import com.huimv.core.annotion.DataScope;
import com.huimv.core.annotion.Permission;
import com.huimv.core.enums.LogAnnotionOpTypeEnum;
import com.huimv.core.pojo.response.ResponseData;
import com.huimv.core.pojo.response.SuccessResponseData;
import com.huimv.modular.analysis.service.AnalysisService;
import com.huimv.modular.stockplan.param.StockPlanParam;
import com.huimv.modular.stockplan.service.StockPlanService;
import lombok.Data;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import javax.annotation.Resource;
import java.util.List;

/**
 * 计划出栏控制器
 *
 * @author yinhao
 * @date 2021-08-22 10:31:41
 */
@RestController
public class StockPlanController {

    @Resource
    private StockPlanService stockPlanService;


    /**
     * 查询计划出栏
     *
     * @author yinhao
     * @date 2021-08-22 10:31:41
     */
    //@Permission
    @DataScope
    @GetMapping("/stockPlan/page")
    @BusinessLog(title = "计划出栏_查询", opType = LogAnnotionOpTypeEnum.QUERY)
    public ResponseData page(StockPlanParam stockPlanParam) {
        return new SuccessResponseData(stockPlanService.page(stockPlanParam));
    }

    /**
     * 添加计划出栏
     *
     * @author yinhao
     * @date 2021-08-22 10:31:41
     */
    //@Permission
    @PostMapping("/stockPlan/add")
    @BusinessLog(title = "计划出栏_增加", opType = LogAnnotionOpTypeEnum.ADD)
    public ResponseData add(@RequestBody @Validated(StockPlanParam.add.class) StockPlanParam stockPlanParam) {
            stockPlanService.add(stockPlanParam);
        return new SuccessResponseData();
    }

    /**
     * 删除计划出栏，可批量删除
     *
     * @author yinhao
     * @date 2021-08-22 10:31:41
     */
    //@Permission
    @PostMapping("/stockPlan/delete")
    @BusinessLog(title = "计划出栏_删除", opType = LogAnnotionOpTypeEnum.DELETE)
    public ResponseData delete(@RequestBody List<Long> stockPlanParamList) {
            stockPlanService.delete(stockPlanParamList);
        return new SuccessResponseData();
    }

    /**
     * 编辑计划出栏
     *
     * @author yinhao
     * @date 2021-08-22 10:31:41
     */
    //@Permission
    @PostMapping("/stockPlan/edit")
    @BusinessLog(title = "计划出栏_编辑", opType = LogAnnotionOpTypeEnum.EDIT)
    public ResponseData edit(@RequestBody @Validated(StockPlanParam.edit.class) StockPlanParam stockPlanParam) {
            stockPlanService.edit(stockPlanParam);
        return new SuccessResponseData();
    }

    /**
     * 查看计划出栏
     *
     * @author yinhao
     * @date 2021-08-22 10:31:41
     */
    //@Permission
    @GetMapping("/stockPlan/detail")
    @BusinessLog(title = "计划出栏_查看", opType = LogAnnotionOpTypeEnum.DETAIL)
    public ResponseData detail(@Validated(StockPlanParam.detail.class) StockPlanParam stockPlanParam) {
        return new SuccessResponseData(stockPlanService.detail(stockPlanParam));
    }

    /**
     * 计划出栏列表
     *
     * @author yinhao
     * @date 2021-08-22 10:31:41
     */
    //@Permission
    @GetMapping("/stockPlan/list")
    @BusinessLog(title = "计划出栏_列表", opType = LogAnnotionOpTypeEnum.QUERY)
    public ResponseData list(StockPlanParam stockPlanParam) {
        return new SuccessResponseData(stockPlanService.list(stockPlanParam));
    }

    /**
     * 导出系统用户
     *
     * @author yinhao
     * @date 2021-08-22 10:31:41
     */
    //@Permission
    @GetMapping("/stockPlan/export")
    @BusinessLog(title = "计划出栏_导出", opType = LogAnnotionOpTypeEnum.EXPORT)
    public void export(StockPlanParam stockPlanParam) {
        stockPlanService.export(stockPlanParam);
    }



    @DataScope
    @GetMapping("/stockPlan/statistic")
    public ResponseData stockAndTargetStockStatistic(StockPlanParam stockPlanParam) {
        return new SuccessResponseData(stockPlanService.statistic(stockPlanParam));
    }

}
