/*
Copyright [2020] [https://www.xiaonuo.vip]

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：

1.请不要删除和修改根目录下的LICENSE文件。
2.请不要删除和修改Snowy源码头部的版权声明。
3.请保留源码和相关描述文件的项目出处，作者声明等。
4.分发源码时候，请注明软件出处 https://gitee.com/xiaonuobase/snowy
5.在修改包名，模块名称，项目代码等时，请注明软件出处 https://gitee.com/xiaonuobase/snowy
6.若您的项目无法满足以上几点，可申请商业授权，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.huimv.modular.stockplan.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.core.context.login.LoginContextHolder;
import com.huimv.core.enums.CommonStatusEnum;
import com.huimv.core.exception.ServiceException;
import com.huimv.core.factory.PageFactory;
import com.huimv.core.pojo.page.PageResult;
import com.huimv.core.util.PoiUtil;
import com.huimv.modular.analysis.entity.RegionAndStock;
import com.huimv.modular.analysis.param.AnalysisParam;
import com.huimv.modular.analysis.service.AnalysisService;
import com.huimv.modular.stockplan.entity.StockAndStockPlan;
import com.huimv.modular.stockplan.entity.StockPlan;
import com.huimv.modular.stockplan.enums.StockPlanExceptionEnum;
import com.huimv.modular.stockplan.mapper.StockPlanMapper;
import com.huimv.modular.stockplan.param.StockPlanParam;
import com.huimv.modular.stockplan.result.StockPlanResult;
import com.huimv.modular.stockplan.service.StockPlanService;
import com.huimv.sys.modular.org.entity.SysOrg;
import com.huimv.sys.modular.org.enums.SysOrgExceptionEnum;
import com.huimv.sys.modular.org.service.SysOrgService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 计划出栏service接口实现类
 *
 * @author yinhao
 * @date 2021-08-22 10:31:41
 */
@Service
public class StockPlanServiceImpl extends ServiceImpl<StockPlanMapper, StockPlan> implements StockPlanService {

    @Resource
    private AnalysisService analysisService;

    @Resource
    private SysOrgService sysOrgService;


    @Override
    public PageResult<StockPlanResult> page(StockPlanParam stockPlanParam) {
        QueryWrapper<StockPlanResult> queryWrapper = Wrappers.query();
        if (ObjectUtil.isNotNull(stockPlanParam.getYear())) {
            queryWrapper.eq("plan.year", stockPlanParam.getYear());
        }
        if (!LoginContextHolder.me().isSuperAdmin()) {
            Long sysLoginUserOrgId = LoginContextHolder.me().getSysLoginUserOrgId();
            List<Long> dataScope = stockPlanParam.getDataScope();
            List<SysOrg> visibleList = sysOrgService.getVisibleListByOrgIdAndDataScope(sysLoginUserOrgId, dataScope);
            if (CollUtil.isNotEmpty(visibleList)) {
                List<Long> collect = visibleList.stream().map(SysOrg::getId).collect(Collectors.toList());
                queryWrapper.in("plan.org_id", collect);
            }
        }
        queryWrapper.orderByDesc("plan.year,plan.target_out");
        Page<StockPlanResult> page = baseMapper.page(PageFactory.defaultPage(), queryWrapper);
        return new PageResult<>(page);
    }

    @Override
    public List<StockPlan> list(StockPlanParam stockPlanParam) {
        return this.list();
    }

    @Override
    public void add(StockPlanParam stockPlanParam) {

        Long orgId = stockPlanParam.getOrgId();
        LambdaQueryWrapper<SysOrg> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(SysOrg::getId, orgId)
                .ne(SysOrg::getStatus, CommonStatusEnum.DELETED.getCode());

        SysOrg sysOrg = sysOrgService.getOne(queryWrapper);
        if (sysOrg == null) {
            throw new ServiceException(SysOrgExceptionEnum.ORG_NOT_EXIST);
        }
        stockPlanParam.setParentOrgId(sysOrg.getPid());
//        if (sysOrg.getPid().equals(0L)) {
//            stockPlanParam.setParentOrgId(0L);
//        } else {
//            stockPlanParam.setParentOrgId(sysLoginUserOrgId);
//        }
        StockPlan stockPlan = new StockPlan();
        BeanUtil.copyProperties(stockPlanParam, stockPlan);
        validateTarget(stockPlanParam);

        this.save(stockPlan);
    }

    private void validateTarget(StockPlanParam stockPlanParam) {

        //组织父id如果为为0,说明是根节点,不做校验
        if (stockPlanParam.getParentOrgId().equals(0L)) {
            return;
        }

        //对剩余可分配年目标出栏量进行校验
        //当前表单输入值
        Integer targetOut = stockPlanParam.getTargetOut();

        //上级组织可分配最大值
        Integer maxTargetOut = baseMapper.selectTargetOutByOrgIdAndYear(stockPlanParam);
        if (ObjectUtil.isNull(maxTargetOut)) {
            throw new ServiceException(StockPlanExceptionEnum.NOT_EXIST);
        }

        //已经分配的总和
        Integer assignedTargetOut = baseMapper.getSumTargetOutByParentOrgIdAndYear(stockPlanParam);
        if (ObjectUtil.isNull(assignedTargetOut)) {
            if (targetOut > maxTargetOut) {
                throw new ServiceException(StockPlanExceptionEnum.OUT_RANGE.getCode(), StockPlanExceptionEnum.OUT_RANGE.getMessage() + ",可输入的最大值为:" + maxTargetOut);
            }
        } else {
            //剩余可分配
            int remainder = maxTargetOut - assignedTargetOut;
            if (remainder < targetOut) {
                throw new ServiceException(StockPlanExceptionEnum.OUT_RANGE.getCode(), StockPlanExceptionEnum.OUT_RANGE.getMessage() + ",可输入的最大值为:" + remainder);
            }
        }

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<Long> stockPlanParamList) {
        this.removeByIds(stockPlanParamList);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(StockPlanParam stockPlanParam) {
        validateTarget(stockPlanParam);
        StockPlan stockPlan = this.queryStockPlan(stockPlanParam);
        BeanUtil.copyProperties(stockPlanParam, stockPlan);
        this.updateById(stockPlan);
    }

    @Override
    public StockPlan detail(StockPlanParam stockPlanParam) {
        return this.queryStockPlan(stockPlanParam);
    }

    /**
     * 获取计划出栏
     *
     * @author yinhao
     * @date 2021-08-22 10:31:41
     */
    private StockPlan queryStockPlan(StockPlanParam stockPlanParam) {
        StockPlan stockPlan = this.getById(stockPlanParam.getId());
        if (ObjectUtil.isNull(stockPlan)) {
            throw new ServiceException(StockPlanExceptionEnum.NOT_EXIST);
        }
        return stockPlan;
    }

    @Override
    public void export(StockPlanParam stockPlanParam) {
        List<StockPlan> list = this.list(stockPlanParam);
        PoiUtil.exportExcelWithStream("SnowyStockPlan.xls", StockPlan.class, list);
    }


    @Override
    public StockAndStockPlan statistic(StockPlanParam stockPlanParam) {
        if (LoginContextHolder.me().isSuperAdmin()) {
            return null;
        }
        Long sysLoginUserOrgId = LoginContextHolder.me().getSysLoginUserOrgId();
        AnalysisParam analysisParam = new AnalysisParam();
        analysisParam.setOrgId(sysLoginUserOrgId);
        analysisParam.setDataScope(stockPlanParam.getDataScope());

        RegionAndStock regionAndStock = analysisService.getChildPigStockByOrgId(analysisParam);
        StockAndStockPlan stockAndStockPlan = new StockAndStockPlan();
        stockAndStockPlan.setRegionList(regionAndStock.getRegionList());
        stockAndStockPlan.setStockList(regionAndStock.getStockList());

        List<Integer> targetStockList = new ArrayList<>();
        List<Double> finishRate = new ArrayList<>();

        LambdaQueryWrapper<StockPlan> lambdaQuery = Wrappers.lambdaQuery();
        int year = DateUtil.year(new Date());
        List<Long> orgIdList = regionAndStock.getOrgIdList();
        for (int i = 0; i < orgIdList.size(); i++) {
            lambdaQuery.eq(StockPlan::getOrgId, orgIdList.get(i))
                    .eq(StockPlan::getYear, year);

            StockPlan stockPlan = getOne(lambdaQuery);
            lambdaQuery.clear();
            if (stockPlan != null) {
                targetStockList.add(stockPlan.getTargetOut());
            } else {
                targetStockList.add(0);
            }

            Integer targetStock = targetStockList.get(i);
            if (targetStock != 0) {
                BigDecimal stock = new BigDecimal(String.valueOf(stockAndStockPlan.getStockList().get(i) * 100));
                BigDecimal target = new BigDecimal(String.valueOf(targetStock));
                finishRate.add(stock.divide(target, 2, BigDecimal.ROUND_HALF_UP).doubleValue());
            } else {
                finishRate.add(0.0);
            }
        }
        stockAndStockPlan.setTargetStockList(targetStockList);
        stockAndStockPlan.setFinishRate(finishRate);
        return stockAndStockPlan;
    }
}
